@@ -6,9 +6,9 @@
 typedef struct pool_str {
   char ip[36];
 
-  /*  int port;*/// not needed
-  int pop3_auth_mechanism;
-  int disable_tls;
+  /*  int32_t port;*/// not needed
+  int32_t pop3_auth_mechanism;
+  int32_t disable_tls;
   struct pool_str *next;
 } pool;
 
@@ -18,7 +18,7 @@ char apop_challenge[300] = "";
 pool *plist = NULL, *p = NULL;
 
 /* functions */
-int service_pop3_init(char *ip, int sp, unsigned char options, char *miscptr, FILE * fp, int port, char *hostname);
+int32_t service_pop3_init(char *ip, int32_t sp, unsigned char options, char *miscptr, FILE * fp, int32_t port, char *hostname);
 
 pool *list_create(pool data) {
   pool *p;
@@ -59,9 +59,9 @@ pool *list_find(char *ip) {
 
 /* how to know when to release the mem ?
    -> well, after _start has determined which pool number it is */
-int list_remove(pool * node) {
+int32_t list_remove(pool * node) {
   pool *save, *list = plist;
-  int ok = -1;
+  int32_t ok = -1;
 
   if (list == NULL || node == NULL)
     return -2;
@@ -78,9 +78,9 @@ int list_remove(pool * node) {
   return ok;
 }
 
-char *pop3_read_server_capacity(int sock) {
+char *pop3_read_server_capacity(int32_t sock) {
   char *ptr = NULL;
-  int resp = 0;
+  int32_t resp = 0;
   char *buf = NULL;
 
   do {
@@ -117,7 +117,7 @@ STLS
   return buf;
 }
 
-int start_pop3(int s, char *ip, int port, unsigned char options, char *miscptr, FILE * fp) {
+int32_t start_pop3(int32_t s, char *ip, int32_t port, unsigned char options, char *miscptr, FILE * fp) {
   char *empty = "\"\"";
   char *login, *pass, buffer[500], buffer2[500], *fooptr;
 
@@ -137,7 +137,7 @@ int start_pop3(int s, char *ip, int port, unsigned char options, char *miscptr,
   case AUTH_APOP:{
       MD5_CTX c;
       unsigned char md5_raw[MD5_DIGEST_LENGTH];
-      int i;
+      int32_t i;
       char *pbuffer = buffer2;
 
       MD5_Init(&c);
@@ -216,7 +216,7 @@ int start_pop3(int s, char *ip, int port, unsigned char options, char *miscptr,
   case AUTH_CRAMMD5:
   case AUTH_CRAMSHA1:
   case AUTH_CRAMSHA256:{
-      int rc = 0;
+      int32_t rc = 0;
       char *preplogin;
 
       rc = sasl_saslprep(login, SASL_ALLOW_UNASSIGNED, &preplogin);
@@ -397,6 +397,8 @@ int start_pop3(int s, char *ip, int port, unsigned char options, char *miscptr,
   }
   /* special AS/400 hack */
   if (strstr(buf, "CPF2204") != NULL || strstr(buf, "CPF22E3") != NULL || strstr(buf, "CPF22E4") != NULL || strstr(buf, "CPF22E5") != NULL) {
+    if (verbose)
+      printf("[INFO] user %s does not exist, skipping\n", login);
     hydra_completed_pair_skip();
     free(buf);
     if (memcmp(hydra_get_next_pair(), &HYDRA_EXIT, sizeof(HYDRA_EXIT)) == 0)
@@ -411,8 +413,8 @@ int start_pop3(int s, char *ip, int port, unsigned char options, char *miscptr,
   return 2;
 }
 
-void service_pop3(char *ip, int sp, unsigned char options, char *miscptr, FILE * fp, int port, char *hostname) {
-  int run = 1, next_run = 1, sock = -1;
+void service_pop3(char *ip, int32_t sp, unsigned char options, char *miscptr, FILE * fp, int32_t port, char *hostname) {
+  int32_t run = 1, next_run = 1, sock = -1;
   char *ptr = NULL;
 
   //extract data from the pool, ip is the key
@@ -438,7 +440,7 @@ void service_pop3(char *ip, int sp, unsigned char options, char *miscptr, FILE *
 
       if (sock >= 0)
         sock = hydra_disconnect(sock);
-      //      sleepn(300);
+      //      usleepn(300);
       if ((options & OPTION_SSL) == 0) {
         sock = hydra_connect_tcp(ip, port);
       } else {
@@ -446,7 +448,7 @@ void service_pop3(char *ip, int sp, unsigned char options, char *miscptr, FILE *
       }
       if (sock < 0) {
         if (verbose || debug)
-          hydra_report(stderr, "[ERROR] Child with pid %d terminating, can not connect\n", (int) getpid());
+          hydra_report(stderr, "[ERROR] Child with pid %d terminating, can not connect\n", (int32_t) getpid());
         hydra_child_exit(1);
       }
       buf = hydra_receive_line(sock);
@@ -511,10 +513,10 @@ void service_pop3(char *ip, int sp, unsigned char options, char *miscptr, FILE *
 }
 
 
-int service_pop3_init(char *ip, int sp, unsigned char options, char *miscptr, FILE * fp, int port, char *hostname) {
-  int myport = PORT_POP3, mysslport = PORT_POP3_SSL;
+int32_t service_pop3_init(char *ip, int32_t sp, unsigned char options, char *miscptr, FILE * fp, int32_t port, char *hostname) {
+  int32_t myport = PORT_POP3, mysslport = PORT_POP3_SSL;
   char *ptr = NULL;
-  int sock = -1;
+  int32_t sock = -1;
   char *capa_str = "CAPA\r\n";
   char *quit_str = "QUIT\r\n";
   pool p;
@@ -523,9 +525,6 @@ int service_pop3_init(char *ip, int sp, unsigned char options, char *miscptr, FI
   p.disable_tls = 1;
   memcpy(p.ip, ip, 36);
 
-  if (sock >= 0)
-    sock = hydra_disconnect(sock);
-//      sleepn(300);
   if ((options & OPTION_SSL) == 0) {
     if (port != 0)
       myport = port;
@@ -537,7 +536,7 @@ int service_pop3_init(char *ip, int sp, unsigned char options, char *miscptr, FI
   }
   if (sock < 0) {
     if (verbose || debug)
-      hydra_report(stderr, "[ERROR] pid %d terminating, can not connect\n", (int) getpid());
+      hydra_report(stderr, "[ERROR] pid %d terminating, can not connect\n", (int32_t) getpid());
     return -1;
   }
   buf = hydra_receive_line(sock);
@@ -572,12 +571,12 @@ int service_pop3_init(char *ip, int sp, unsigned char options, char *miscptr, FI
   }
 
   if ((miscptr != NULL) && (strlen(miscptr) > 0)) {
-    int i;
+    int32_t i;
 
     for (i = 0; i < strlen(miscptr); i++)
-      miscptr[i] = (char) toupper((int) miscptr[i]);
+      miscptr[i] = (char) toupper((int32_t) miscptr[i]);
 
-    if (strstr(miscptr, "TLS") || strstr(miscptr, "SSL")) {
+    if (strstr(miscptr, "TLS") || strstr(miscptr, "SSL") || strstr(miscptr, "STARTTLS")) {
       p.disable_tls = 0;
     }
   }
@@ -776,3 +775,9 @@ int service_pop3_init(char *ip, int sp, unsigned char options, char *miscptr, FI
 
   return 0;
 }
+
+void usage_pop3(const char* service) {
+  printf("Module pop3 is optionally taking one authentication type of:\n"
+         "  CLEAR (default), LOGIN, PLAIN, CRAM-MD5, CRAM-SHA1,\n"
+         "  CRAM-SHA256, DIGEST-MD5, NTLM.\n" "Additionally TLS encryption via STLS can be enforced with the TLS option.\n\n" "Example: pop3://target/TLS:PLAIN\n");
+}
