@@ -19,12 +19,12 @@ void dummy_sshkey() {
 
 extern ssh_session session;
 extern char *HYDRA_EXIT;
-extern int new_session;
+extern int32_t new_session;
 
-int start_sshkey(int s, char *ip, int port, unsigned char options, char *miscptr, FILE * fp) {
+int32_t start_sshkey(int32_t s, char *ip, int32_t port, unsigned char options, char *miscptr, FILE * fp) {
   char *empty = "";
   char *login, *key, keep_login[300];
-  int auth_state = 0, rc = 0;
+  int32_t auth_state = 0, rc = 0;
   ssh_private_key privkey;
 
   if (strlen(login = hydra_get_next_login()) == 0)
@@ -108,8 +108,8 @@ int start_sshkey(int s, char *ip, int port, unsigned char options, char *miscptr
   return 1;
 }
 
-void service_sshkey(char *ip, int sp, unsigned char options, char *miscptr, FILE * fp, int port, char *hostname) {
-  int run = 1, next_run = 1, sock = -1;
+void service_sshkey(char *ip, int32_t sp, unsigned char options, char *miscptr, FILE * fp, int32_t port, char *hostname) {
+  int32_t run = 1, next_run = 1, sock = -1;
 
   hydra_register_socket(sp);
   if (memcmp(hydra_get_next_pair(), &HYDRA_EXIT, sizeof(HYDRA_EXIT)) == 0)
@@ -151,7 +151,7 @@ void service_sshkey(char *ip, int sp, unsigned char options, char *miscptr, FILE
 #endif
 #endif
 
-int service_sshkey_init(char *ip, int sp, unsigned char options, char *miscptr, FILE * fp, int port, char *hostname) {
+int32_t service_sshkey_init(char *ip, int32_t sp, unsigned char options, char *miscptr, FILE * fp, int32_t port, char *hostname) {
   // called before the childrens are forked off, so this is the function
   // which should be filled if initial connections and service setup has to be
   // performed once only.
@@ -164,3 +164,10 @@ int service_sshkey_init(char *ip, int sp, unsigned char options, char *miscptr,
 
   return 0;
 }
+
+void usage_sshkey(const char* service) {
+  printf("Module sshkey does not provide additional options, although the semantic for\n"
+         "options -p and -P is changed:\n"
+         "  -p expects a path to an unencrypted private key in PEM format.\n"
+         "  -P expects a filename containing a list of path to some unencrypted\n" "     private keys in PEM format.\n\n");
+}
